import pickle, time

calendar_file = 'index.html'
events_file = 'events'
archive_file = 'archive'

DEFAULT_WEEKS = 16

def _format_entry(event):
  return event.replace('\r', '').rstrip('\n').replace('\n', '\n<br>\n')


def _key(yr, mo, day):
  return '%04d%02d%02d' % (yr, mo, day)


def _get_month_name(mnth):
  return ('', 'Jan', 'Feb', 'March', 'April', 'May', 'June', \
          'July', 'Aug', 'Sept', 'Oct', 'Nov', 'Dec')[mnth]


def _get_days(mnth, year=time.localtime()[0]):
  """Return the number of days in the month and year given
     (default is current year).
  """
  if mnth == 2:
    if year%4 == 0 and not (year%100 == 0 and year%400 > 0):
      return 29
    else:
      return 28
  elif (mnth < 8 and mnth%2 == 0) or (mnth > 8 and mnth%2 == 1):
    return 30
  else:
    return 31


def _get_first_day(month, year=time.localtime()[0]):
  """Return the numerical day of the week (0-6, 0=Sunday) the first day of 
     the given month in the given year (default is current year) falls on.
  """
  day_tuple = time.localtime(time.mktime((year,month,1,0,0,0,0,0,0)))
  return (day_tuple[6]+1) % 7


def _step_month(month, year, step):
  month += step
  while month > 12:
    month -= 12
    year += 1
  while month < 1:
    month += 12
    year -= 1
  return (month, year)

#~~~~~~~~~~ write calendar page (helper for update_pages) ~~~~~~~~~~~#

def create_calendar(event_dict, weeks, month, yr, edit):
  """Write a calendar to the file given by urlindex for the month and year
     given by 'currmonth' and 'year', populating the days with events
     from the dictionary of events given in event_dict.
  """
  width = 745
  font_fam = 'arial narrow, arial, helvetica'
  height = 115
  font_size = '8pt'
  method = 'get'
  if edit:
    method = 'post'
  string = '''<html>
<head>
<title>Calendar</title>
<style>
  a:link    {color: #993333;}
  a:visited {color: #666666;}
  a:active  {color: #ff3333;}
</style>
</head>
<body>
<table width="''' + str(width) + '''"
       border="1"
       cellpadding="2"
       cellspacing="0"
       style="border-relief:flat; border-color:#cccccc;">
<form action="index.cgi" method="''' + method + '">\n'

  currmonth = month
  year = yr
  emptycells = _get_first_day(currmonth, year)
  if emptycells:
    (currmonth, year) = _step_month(currmonth, year, -1)
    currentday = _get_days(currmonth, year) - emptycells + 1
  else:
    currentday = 1
  days = _get_days(currmonth, year)
  (thisyear, thismonth, thisday) = time.localtime()[:3]
  
  (nav_back_month, nav_back_year) = _step_month(month, yr, -1)
  (nav_forward_month, nav_forward_year) = _step_month(month, yr, 1)
  nav_back = ' <a href="index.cgi?weeks='+str(DEFAULT_WEEKS)+\
             '&month='+str(nav_back_month)+\
             '&year='+str(nav_back_year)+\
             '&edit='+str(edit)+'">&lt;&lt;</a> '
  nav_forward = ''

  for i in range(weeks):
    string += '<tr>\n'
    for j in range(1, 8):
      if i == weeks-1 and j == 8-1:
        nav_forward = ' <a href="index.cgi?weeks='+str(DEFAULT_WEEKS)+\
                      '&month='+str(nav_forward_month)+\
                      '&year='+str(nav_forward_year)+\
                      '&edit='+str(edit)+'">&gt;&gt;</a> '
      if _key(year, currmonth, currentday) < _key(thisyear, thismonth, thisday):
        color = "#eeeeee"
      else:
        color = "#ffffff"
      string += '  <td width="14%" height="' + str(height) + '''" 
      valign="top" align="left"
      bgcolor=''' + color + '''>
    <table width="100%"
           cellpadding="0"
           cellspacing="0"
           style="font-family:''' + font_fam + '''; font-size:''' + font_size + ''';">
    <tr>
      <td valign="top" align="left">''' + nav_back
      if currentday == 1:
        string +=  _get_month_name(currmonth) + ' '
      string += str(currentday) + '<br>'
      nav_back = ''
      string += '''</td>
      <td valign="top" align="right">''' + nav_forward + '''</td>
    <tr>
      <td colspan="2" valign="top" align="left">'''
      key = _key(year, currmonth, currentday)
      if edit:
        string += '''
        <br>
        <textarea name="''' + key + '''" rows=6 
                  style="width: 100; font-family:''' + \
                  font_fam + '''; font-size:''' + \
                  font_size + ''';">\n''' + \
        event_dict.get(key, '\n\n\n\n\n\n') + '''</textarea>
        <input type="hidden" name="modified" value="''' + key + '">'
      else:
        string += _format_entry(event_dict.get(key, ''))
      string += '''
      </td>
    </tr>
    </table>
  </td>\n'''
      currentday += 1
      if currentday > days:
        (currmonth, year) = _step_month(currmonth, year, 1)
        days = _get_days(currmonth, year)
        currentday = 1
  sel_weeks = {DEFAULT_WEEKS: ' selected="selected"'}
  sel_months = {month: ' selected="selected"'}
  sel_years = {yr: ' selected="selected"'}
  string += '''</tr>
</table>
<table>
<!--<tr>
  <td>
    <textarea name="misc" rows=6
              style="width: ''' + str(width) + '''; font-family:''' + \
              font_fam + '''; font-size:''' + \
              font_size + ''';">\n''' + \
    event_dict.get('misc', '') + '''</textarea>
  </td>
</tr>-->
<tr>
  <td><font face="arial" size="-1">
    Weeks:
    <select name="weeks" size="1">\n'''
  for i in range(4, 26):
    string += '      <option value="%d"%s>%d\n' % (i, sel_weeks.get(i, ''), i)
  string += '''    </select>
    Start month:
    <select name="month" size="1">\n'''
  for i in range(1, 13):
    string += '      <option value="%d"%s>%d\n' % (i, sel_months.get(i, ''), i)
  string += '''    </select>
    Year:
    <select name="year" size="1">\n'''
  for i in range(time.localtime()[0]-1, time.localtime()[0]+2):
    string += '      <option value="%d"%s>%d\n' % (i, sel_years.get(i, ''), i)
  string += '    </select>\n'
  if edit:
    string += '    <input type="checkbox" name="edit" value="0" checked> Normal view\n'
    string += '    <input type="hidden" name="edit" value="1">'
  else:
    string += '    <input type="checkbox" name="edit" value="1" checked> Edit view'
  string += '''
    <input type="submit" value="Update">
  </font>
  </td>
</tr>'''
  string += '''</form>
</table>'''

  return string

#~~~~~~~~~~~~~~~~~~~~~~~~~~~ save events ~~~~~~~~~~~~~~~~~~~~~~~~~#

def save_events(dict, modified):
  file = open(events_file)
  events = pickle.load(file)
  file.close()
  file = open(archive_file)
  archive = pickle.load(file)
  file.close()
  for event in modified:
    events[event] = dict.get(event, '')
    archive[event] = dict.get(event, '')
  file = open(events_file, 'w')
  pickle.dump(events, file)
  file.close()
  file = open(archive_file, 'w')
  pickle.dump(archive, file)
  file.close()
  

def archive_events():
  file = open(events_file)
  events = pickle.load(file)
  file.close()
  date = time.localtime()
  key = key(date[0], date[1], 1)
  for event in events.keys():
    if event < key:
      del events[event]
  file = open(events_file, 'w')
  pickle.dump(events, file)
  file.close()
  

def main(dict):
  weeks = int(dict.get('weeks', [DEFAULT_WEEKS])[0])
  month = int(dict.get('month', [time.localtime()[1]])[0])
  year = int(dict.get('year', [time.localtime()[0]])[0])
  edit = int(dict.get('edit', [0])[0])
  modified = dict.get('modified', [])
  svdict = {}
  
  if 'weeks' in dict: 
    del dict['weeks']
  if 'month' in dict:
    del dict['month']
  if 'year' in dict:
    del dict['year']
  if 'edit' in dict:
    del dict['edit']
  if 'modified' in dict:
    del dict['modified']

  for key in dict.keys():
    svdict[key] = dict[key][0]
  save_events(svdict, modified)

  if month < time.localtime()[1] or year < time.localtime()[0]:
    file = open(archive_file)
    events = pickle.load(file)
    file.close()
  else:
    file = open(events_file)
    events = pickle.load(file)
    file.close()

  return create_calendar(events, weeks, month, year, edit)
